; ============================================================
; Programmable automatic turn signal
;
; The code is controlled by pressing the button 1 to 6 times,
; the last press may be short or long:
;
; count	short	long	action
; -----	-----	----	------
; 1	get	set	blink count (0..5, default: 3)
; 2	get	set	blink interval (0..8, default: 3)
; 3	get	set	activation pulse width (0..8, default: 3)
; 4	get	set	debug flag (0..3, default: 0)
; 5	-	start	alternating signal
; 6	-	start	SOS signal
;
; A "get" action will show the value as a number of LED flashes.
; After a "set" action, the new value must be entered by quickly
; pressing the button the desired number of times.
;
; meaning of the debug flag:
; 0	no debug info
; 1	reset signal
; 2	reset signal + signal when BLINKS > 0
; 3	reset signal + signal when MOSFETs activated
;
; Note: Fuse byte of the ATtiny12 set to 0x12 to enable the BOD.
;
; Copyright (C) 2005, 2006 Dick Streefland
;
; This is free software, licensed under the terms of the GNU General
; Public License as published by the Free Software Foundation.
; ============================================================

.equ	CLOCK,		1200000	; internal clock frequency (Hz)
.equ	RATE,		250	; timer interrupt rate (Hz)
.equ	DEF_COUNT,	3	; default for COUNT
.equ	MAX_COUNT,	5	; maximum for COUNT
.equ	DEF_INTERVAL,	3	; default for INTERVAL
.equ	MAX_INTERVAL,	8	; maximum for INTERVAL
.equ	DEF_PULSE,	3	; default for PULSE
.equ	MAX_PULSE,	8	; maximum for PULSE
.equ	DEF_DEBUG,	0	; default for DEBUG
.equ	MAX_DEBUG,	3	; maximum for DEBUG
.equ	EEPROM_COUNT,	0	; EEPROM address for COUNT
.equ	EEPROM_INTERVAL,1	; EEPROM address for INTERVAL
.equ	EEPROM_PULSE,	2	; EEPROM address for PULSE
.equ	EEPROM_DEBUG,	3	; EEPROM address for DEBUG

#define	MSEC2TICKS(msec)	(RATE * (msec) / 1000)

; the following times should not exceed 255 ticks (255000/RATE):
.equ	IDLE_TIME,	500	; idle time before activation pulse (msec)
.equ	STUCK_LIMIT,	800	; maximum on time of relais
.equ	PULSE_BASE,	128	; base width of activation pulse (msec)
.equ	PULSE_INCR,	32	; base width of activation pulse (msec)
.equ	INTERVAL_BASE,	290	; interval base (msec)
.equ	INTERVAL_INCR,	40	; interval increment (msec)
.equ	BUTTON_LONG,	500	; minimum time of long button press (msec)
.equ	BUTTON_TIMEOUT,	700	; button timeout (msec)
.equ	LED_ON,		150	; LED on time (msec)
.equ	LED_OFF,	250	; LED off time (msec)
.equ	SOS_SHORT,	200	; width of short morse pulse
.equ	SOS_LONG,	400	; width of long morse pulse
.equ	SOS_PAUSE,	800	; pause between words

#define	USEC2CYCLES(usec)	((usec) * CLOCK / 1000000)

; the switch-off time must be at least 30 usec:
.equ	POLL_TIME,	100	; switch-off time to poll switch (usec)

.equ	PINB,	0x16
.equ	DDRB,	0x17
.equ	PORTB,	0x18
.equ	 SIGNAL_R,	0
.equ	 SIGNAL_L,	1
.equ	 SPARE,		2
.equ	 MOSFET_L,	3
.equ	 MOSFET_R,	4
.equ	EECR,	0x1c		; EEPROM Control Register
.equ	 EEMWE,	2		; - Master Write Enable
.equ	 EEWE,	1		; - Write Enable
.equ	 EERE,	0		; - Read Enable
.equ	EEDR,	0x1d		; EEPROM Data Register
.equ	EEAR,	0x1e		; EEPROM Address Register
.equ	WDTCR,	0x21		; Watchdog Timer Control Register
.equ	TCNT0,	0x32		; Timer Counter 0
.equ	TCCR0,	0x33		; Timer/Counter0 Control Register
.equ	TIMSK,	0x39		; Timer/Counter Interrupt Mask Register
.equ	SREG,	0x3f		; Status Register

; ------------------------------------------------------------
; Register definitions
; ------------------------------------------------------------
#define	LED		r0	// state of LED
#define	LEFT		r1	// left light  (0x00=off 0xff=on other=bouncing)
#define	RIGHT		r2	// right light (0x00=off 0xff=on other=bouncing)
#define	BUTTON		r3	// button      (0x00=off 0xff=on other=bouncing)
#define	TICKS		r4	// counts timer interrupts (saturating)
#define	INTERVAL	r5	// interval width
#define	PULSE		r6	// pulse width
#define	BLINKS		r7	// interrupt: number of remaining blinks
#define	SIDE		r8	// interrupt: DEBOUNCED at start of auto-blink
#define	SREG_SAVE	r9	// interrupt: SREG save
#define	PORTB_SAVE	r10	// interrupt: PORTB save
#define	DISABLE		r11	// disable auto-blink

#define	TEMP		r16	// temp for use outside the interrupt handler
#define	ITEMP		r17	// interrupt: temp
#define	LOW		r18	// interrupt: low period
#define	HIGH		r19	// interrupt: high period
#define	DEBOUNCED	r20	// debounced input flags
#define	VALUE		r21	//
#define	RESULT		r22	//
#define	MAX		r23	//
#define	TIMEOUT		r24	//
#define	DELAY		r25	//
#define	COMMAND		r26	//
#define	COUNT		r27	// 
#define	IDLE		r28	// 
#define	ITICKS		r29	// 
#define	ADDRESS		r30	// EEPROM address
#define	DEBUG		r31	// debug flag

.section .text
; ------------------------------------------------------------
; Vector table
; ------------------------------------------------------------
	rjmp	reset		; RESET
	reti			; External interrupt request 0
	reti			; I/O pins
	rjmp	timer		; Timer/counter 0 overflow
	reti			; EEPROM ready
	reti			; Analog comparator

; ------------------------------------------------------------
; Program start
; ------------------------------------------------------------
reset:
	; initialize I/O pins
	clr	DDRB
	clr	PORTB
	sbi	DDRB, SPARE	; output
	sbi	DDRB, MOSFET_L	; output
	sbi	DDRB, MOSFET_R	; output

	; initialize data
	clr	LEFT
	clr	RIGHT
	clr	LED
	clr	BUTTON
	clr	BLINKS
	ldi	LOW, 1
	clr	HIGH
	clr	DEBOUNCED
	clr	SIDE
	clr	DISABLE

	; initialize COUNT
	ldi	ADDRESS, EEPROM_COUNT
	rcall	eeprom_read
	in	COUNT, EEDR
	cpi	COUNT, MAX_COUNT+1
	brlo	1f
	ldi	COUNT, DEF_COUNT ; fallback to default
1:

	; initialize INTERVAL
	ldi	ADDRESS, EEPROM_INTERVAL
	rcall	eeprom_read
	in	TEMP, EEDR
	cpi	TEMP, MAX_INTERVAL+1
	brlo	1f
	ldi	TEMP, DEF_INTERVAL ; fallback to default
1:
	mov	INTERVAL, TEMP

	; initialize PULSE
	ldi	ADDRESS, EEPROM_PULSE
	rcall	eeprom_read
	in	TEMP, EEDR
	cpi	TEMP, MAX_PULSE+1
	brlo	1f
	ldi	TEMP, DEF_PULSE	; fallback to default
1:
	mov	PULSE, TEMP

	; initialize DEBUG
	ldi	ADDRESS, EEPROM_DEBUG
	rcall	eeprom_read
	in	TEMP, EEDR
	cpi	TEMP, MAX_DEBUG+1
	brlo	1f
	ldi	TEMP, DEF_DEBUG	; fallback to default
1:
	mov	DEBUG, TEMP

	; initializer timer
	ldi	TEMP, 0x03
	out	TCCR0, TEMP	; set prescaler to 64
	ldi	TEMP, 0x02
	out	TIMSK, TEMP	; enable timer interrupt
	sei			; enable all interrupts

	; enable watchdog
	ldi	TEMP, 0x0f	; prescaler 2048K ==> 1.9 seconds
	out	WDTCR, TEMP	; enable watchdog

; ------------------------------------------------------------
; debug code
; ------------------------------------------------------------
	tst	DEBUG
	breq	0f
	com	LED
	ldi	DELAY, MSEC2TICKS(100)
	rcall	delay
	com	LED
	ldi	DELAY, MSEC2TICKS(100)
	rcall	delay
	com	LED
	ldi	DELAY, MSEC2TICKS(100)
	rcall	delay
	com	LED
0:

; ------------------------------------------------------------
; Main loop to watch the button and control the LED
; ------------------------------------------------------------
main:
	clr	LED
0:	rcall	wait_release
	brne	0b
1:	rcall	wait_press
	brne	1b
	clr	COMMAND
2:	inc	COMMAND
	rcall	wait_release
	brne	long_command
	rcall	wait_press
	brne	short_command
	rjmp	2b

short_command:
	mov	VALUE, COUNT
	cpi	COMMAND, 1
	breq	do_show
	mov	VALUE, INTERVAL
	cpi	COMMAND, 2
	breq	do_show
	mov	VALUE, PULSE
	cpi	COMMAND, 3
	breq	do_show
	mov	VALUE, DEBUG
	cpi	COMMAND, 4
	breq	do_show
	rjmp	main
do_show:
	rcall	show
	rjmp	main

long_command:
	cpi	COMMAND, 1
	brne	1f
	ldi	MAX, MAX_COUNT
	rcall	get
	mov	COUNT, RESULT
	ldi	ADDRESS, EEPROM_COUNT
	rcall	eeprom_write
1:
	cpi	COMMAND, 2
	brne	1f
	ldi	MAX, MAX_INTERVAL
	rcall	get
	mov	INTERVAL, RESULT
	ldi	ADDRESS, EEPROM_INTERVAL
	rcall	eeprom_write
1:
	cpi	COMMAND, 3
	brne	1f
	ldi	MAX, MAX_PULSE
	rcall	get
	mov	PULSE, RESULT
	ldi	ADDRESS, EEPROM_PULSE
	rcall	eeprom_write
1:
	cpi	COMMAND, 4
	brne	1f
	ldi	MAX, MAX_DEBUG
	rcall	get
	mov	DEBUG, RESULT
	ldi	ADDRESS, EEPROM_DEBUG
	rcall	eeprom_write
1:
	cpi	COMMAND, 5
	brne	1f
	rjmp	wiggle
1:
	cpi	COMMAND, 6
	brne	1f
	rjmp	sos
1:
	rjmp	main

; ------------------------------------------------------------
wiggle:
	com	LED
0:	rcall	wait_release
	brne	0b
	com	LED
	inc	DISABLE
wiggle_loop:
	sbi	PORTB, MOSFET_L	; left MOSFET on
0:	sbrs	DEBOUNCED, SIGNAL_L
	rjmp	0b
1:	sbrc	DEBOUNCED, SIGNAL_R
	rjmp	wiggle_exit
	sbrc	DEBOUNCED, SIGNAL_L
	rjmp	1b
	cbi	PORTB, MOSFET_L	; left MOSFET off
	sbi	PORTB, MOSFET_R	; right MOSFET on
0:	tst	BUTTON
	brne	wiggle_exit
	sbrs	DEBOUNCED, SIGNAL_R
	rjmp	0b
1:	sbrc	DEBOUNCED, SIGNAL_L
	rjmp	wiggle_exit
	sbrc	DEBOUNCED, SIGNAL_R
	rjmp	1b
	cbi	PORTB, MOSFET_R	; right MOSFET off
	wdr
	tst	BUTTON
	breq	wiggle_loop
wiggle_exit:
	cbi	PORTB, MOSFET_L	; left MOSFET off
	cbi	PORTB, MOSFET_R	; right MOSFET off
	com	LED
0:	sbrc	DEBOUNCED, SIGNAL_L
	rjmp	0b		; button cannot be read when LEFT != 0
0:	rcall	wait_release
	brne	0b
	com	LED
	clr	DISABLE
	rjmp	main

; ------------------------------------------------------------
sos:
	com	LED
0:	rcall	wait_release
	brne	0b
	com	LED
	inc	DISABLE
sos_loop:
	ldi	DELAY,  MSEC2TICKS(SOS_PAUSE)
	rcall	bdelay
	brne	sos_exit
	ldi	DELAY,  MSEC2TICKS(SOS_SHORT)
	rcall	sos3
	brne	sos_exit
	ldi	DELAY,  MSEC2TICKS(SOS_LONG)
	rcall	sos3
	brne	sos_exit
	ldi	DELAY,  MSEC2TICKS(SOS_SHORT)
	rcall	sos3
	breq	sos_loop
sos_exit:
	clr	DISABLE
	rjmp	main

sos3:
	rcall	bdelay
	ldi	TEMP, 3
sos3_loop:
	wdr
	sbi	PORTB, MOSFET_L	; left MOSFET on
	sbi	PORTB, MOSFET_R	; right MOSFET on
	rcall	bdelay
	brne	sos3_exit
	cbi	PORTB, MOSFET_L	; left MOSFET off
	cbi	PORTB, MOSFET_R	; right MOSFET off
	rcall	bdelay
	brne	sos3_exit
	dec	TEMP
	brne	sos3_loop
sos3_exit:
	tst	BUTTON
	ret

bdelay:
	clr	TICKS
0:	tst	BUTTON
	brne	bdelay_exit
	cp	TICKS, DELAY
	brlo	0b
bdelay_exit:
	tst	BUTTON
	ret

; ------------------------------------------------------------
; Wait for button press, with a timeout of BUTTON_TIMEOUT.
; Z==1 when a button press was detected, Z==0 on a timeout. 
; ------------------------------------------------------------
wait_press:
	ldi	TIMEOUT, MSEC2TICKS(BUTTON_TIMEOUT)
	ser	VALUE
	rjmp	wait_for

; ------------------------------------------------------------
; Wait for button release, with a timeout of BUTTON_LONG.
; Z==1 when a button release was detected, Z==0 on a timeout. 
; ------------------------------------------------------------
wait_release:
	ldi	TIMEOUT, MSEC2TICKS(BUTTON_LONG)
	clr	VALUE
	rjmp	wait_for

wait_for:
	wdr
	clr	TICKS
1:	cp	BUTTON, VALUE
	breq	2f
	cp	TICKS, TIMEOUT
	brlo	1b
	clz
2:	ret

; ------------------------------------------------------------
; Get a new value in the range 0..MAX and return it in RESULT.
; ------------------------------------------------------------
get:
	com	LED		; LED is on when a new value is being read
	clr	RESULT
get_loop:
0:	rcall	wait_release
	brne	0b
	rcall	wait_press
	brne	get_done
	cp	RESULT, MAX
	brge	get_loop
	com	LED
	inc	RESULT
0:	rcall	wait_release
	brne	0b
	com	LED
	rjmp	get_loop
get_done:
	com	LED		; LED off
	ret

; ------------------------------------------------------------
; Show VALUE by flashing the LED.
; ------------------------------------------------------------
show_loop:
	wdr
	com	LED
	ldi	DELAY, MSEC2TICKS(LED_ON)
	rcall	delay
	com	LED
	ldi	DELAY, MSEC2TICKS(LED_OFF)
	rcall	delay
show:
	dec	VALUE
	brpl	show_loop
	ret

; ------------------------------------------------------------
; Delay DELAY ticks
; ------------------------------------------------------------
delay:
	clr	TICKS
0:	cp	TICKS, DELAY
	brlo	0b
	ret

; ------------------------------------------------------------
; Read a byte from EEPROM address ADDRESS
; ------------------------------------------------------------
eeprom_read:
	out	EEAR, ADDRESS
0:	sbic	EECR, EEWE
	rjmp	0b		; wait for pending write
	sbi	EECR, EERE	; read byte into EEDR
	ret

; ------------------------------------------------------------
; Write the byte RESULT to EEPROM address ADDRESS
; ------------------------------------------------------------
eeprom_write:
0:	sbic	EECR, EEWE
	rjmp	0b		; wait for pending write
	out	EEAR, ADDRESS
	sbi	EECR, EERE	; read old value into EEDR
	in	TEMP, EEDR
	cp	TEMP, RESULT
	breq	1f		; skip write of same value
	out	EEDR, RESULT
	in	TEMP, SREG
	cli			; disable interrupts for the next 2 instructions
	sbi	EECR, EEMWE
	sbi	EECR, EEWE
	out	SREG, TEMP
1:
	ret

; ------------------------------------------------------------
; Timer interrupt handler
; ------------------------------------------------------------
timer:
	ldi	ITEMP, 256-(CLOCK/64/RATE)
	out	TCNT0, ITEMP	; reload counter
	in	SREG_SAVE, SREG	; save SREG

	; discharge inputs
	cbi	PORTB, SIGNAL_R	; set R low (discharge)
	cbi	PORTB, SIGNAL_L	; set L low (discharge)
	nop

	; poll right signal
	cbi	DDRB, SIGNAL_R	; switch R to input - no pullup
	lsl	RIGHT
	nop
	sbic	PINB, SIGNAL_R
	inc	RIGHT		; shift one bit into RIGHT

	; poll left signal
	cbi	DDRB, SIGNAL_L	; switch L to input - no pullup
	lsl	LEFT
	nop
	sbic	PINB, SIGNAL_L
	inc	LEFT		; shift one bit into LEFT

	; create debounced input flags in DEBOUNCED
	com	RIGHT
	brne	1f
	sbr	DEBOUNCED, (1 << SIGNAL_R)
1:	com	RIGHT
	brne	2f
	cbr	DEBOUNCED, (1 << SIGNAL_R)
2:	com	LEFT
	brne	1f
	sbr	DEBOUNCED, (1 << SIGNAL_L)
1:	com	LEFT
	brne	2f
	cbr	DEBOUNCED, (1 << SIGNAL_L)
2:

	; if DEBOUNCED.SIGNAL_R || DEBOUNCED.SIGNAL_L:
	;	if HIGH == 0 && COUNT > 0 && ! DISABLE:
	;		L/R MOSFET on
	; 		IDLE = LOW
	; 		if DEBUG == 3:
	; 			LED on
	; 	LOW = 0
	; 	if HIGH < STUCK_LIMIT:
	; 		HIGH++
	; 	else:
	; 		MOSFETs off
	; 		if DEBUG == 3:
	; 			LED off
	; 	if DEBOUNCED != SIDE:
	; 		BLINKS = 0
	; 		if DEBUG == 2:
	; 			LED off
	; 	if COUNT > 0
	; 	&& ! DISABLE
	; 	&& BLINKS == 0
	; 	&& IDLE >= IDLE_TIME
	; 	&& HIGH == PULSE_BASE + PULSE_INCR*PULSE msec:
	; 		save PORTB
	; 		switch MOSFETs off
	; 		delay
	; 		if ! (PINB.SIGNAL_R || PINB.SIGNAL_L):
	; 			BLINKS = COUNT
	; 			if DEBUG == 2:
	; 				LED on
	; 			SIDE = DEBOUNCED
	; 		restore PORTB
	tst	DEBOUNCED
	breq	7f
	tst	HIGH
	brne	1f
	tst	COUNT
	breq	1f
	tst	DISABLE
	brne	1f
	sbrc	DEBOUNCED, SIGNAL_L
	sbi	PORTB, MOSFET_L	; left MOSFET on
	sbrc	DEBOUNCED, SIGNAL_R
	sbi	PORTB, MOSFET_R	; right MOSFET on
	mov	IDLE, LOW
	cpi	DEBUG, 3
	brne	1f
	clr	LED
	com	LED		; LED on
1:
	clr	LOW
	inc	HIGH
	cpi	HIGH, MSEC2TICKS(STUCK_LIMIT)
	brlo	2f
	dec	HIGH		; stuck relais?
	cbi	PORTB, MOSFET_L	; switch off MOSFET
	cbi	PORTB, MOSFET_R	; switch off MOSFET
	cpi	DEBUG, 3
	brne	2f
	clr	LED		; LED off
2:
	cp	DEBOUNCED, SIDE
	breq	3f
	clr	BLINKS		; side switch ==> abort auto-blinking
	cpi	DEBUG, 2
	brne	3f
	clr	LED		; LED off
3:
	tst	COUNT
	breq	7f
	tst	DISABLE
	brne	7f
	tst	BLINKS
	brne	7f
	cpi	IDLE, MSEC2TICKS(IDLE_TIME)
	brlo	7f
	mov	ITEMP, PULSE
	ldi	ITICKS, MSEC2TICKS(PULSE_BASE - PULSE_INCR)
4:
	subi	ITICKS, - MSEC2TICKS(PULSE_INCR)
	dec	ITEMP
	brpl	4b
	cp	HIGH, ITICKS
	brne	7f
	in	PORTB_SAVE, PORTB
	cbi	PORTB, MOSFET_L	; switch off MOSFET
	cbi	PORTB, MOSFET_R	; switch off MOSFET
	ldi	ITEMP, USEC2CYCLES(POLL_TIME)/3
5:
	dec	ITEMP
	brne	5b
	in	ITEMP, PINB
	andi	ITEMP, (1 << SIGNAL_L) | (1 << SIGNAL_R)
	brne	6f
	mov	BLINKS, COUNT	; start of auto-blinking
	mov	SIDE, DEBOUNCED
	cpi	DEBUG, 2
	brne	6f
	clr	LED
	com	LED		; LED on
6:
	out	PORTB, PORTB_SAVE
7:

	; if ! (DEBOUNCED.SIGNAL_R || DEBOUNCED.SIGNAL_L):
	; 	if LOW == 0:
	; 		if --BLINKS == 0:
	; 			LOW = IDLE_TIME		// be ready for next cycle
	; 		if BLINKS <= 0:
	; 			MOSFETs off
	; 			BLINKS = 0
	; 			if DEBUG >= 2:
	; 				LED off
	; 		if INTERVAL > 0:
	; 			MOSFETs off
	; 			if DEBUG == 3:
	; 				LED off
	; 	if LOW < 255:
	; 		LOW++
	; 	HIGH = 0
	; 	if INTERVAL > 0 && BLINKS > 0:
	; 		if LOW == INTERVAL_BASE + INTERVAL_INCR*INTERVAL msec:
	;			L/R MOSFET on
	; 			if DEBUG == 3:
	; 				LED on
	tst	DEBOUNCED
	brne	6f
	tst	LOW
	brne	3f
	dec	BLINKS
	brne	0f
	ldi	LOW, MSEC2TICKS(IDLE_TIME)
0:
	breq	1f
	brpl	2f
1:
	cbi	PORTB, MOSFET_L	; switch off MOSFET
	cbi	PORTB, MOSFET_R	; switch off MOSFET
	clr	BLINKS		; end of auto-blinking
	cpi	DEBUG, 2
	brlo	2f
	clr	LED		; LED off
2:
	tst	INTERVAL
	breq	3f
	cbi	PORTB, MOSFET_L	; switch off MOSFET
	cbi	PORTB, MOSFET_R	; switch off MOSFET
	cpi	DEBUG, 3
	brne	3f
	clr	LED		; LED off
3:
	inc	LOW
	brne	4f
	dec	LOW
4:
	clr	HIGH
	tst	INTERVAL
	breq	6f
	tst	BLINKS
	breq	6f
	mov	ITEMP, INTERVAL
	ldi	ITICKS, MSEC2TICKS(INTERVAL_BASE)
5:
	subi	ITICKS, - MSEC2TICKS(INTERVAL_INCR)
	dec	ITEMP
	brne	5b
	cp	ITICKS, LOW
	brne	6f
	sbrc	SIDE, SIGNAL_L
	sbi	PORTB, MOSFET_L	; left MOSFET on
	sbrc	SIDE, SIGNAL_R
	sbi	PORTB, MOSFET_R	; right MOSFET on
	cpi	DEBUG, 3
	brne	6f
	clr	LED
	com	LED		; LED on
6:

	; update ticks, saturating at 255
	ser	ITEMP
	cpse	ITEMP,TICKS
	inc	TICKS

	; poll button
	sbi	DDRB, SIGNAL_L	; left "input" is output for a moment ...
	cbi	PORTB, SIGNAL_L	; ... to discharge the input
	nop
	sbi	DDRB, SIGNAL_R	; right "input" is output ...
	sbi	PORTB, SIGNAL_R	; ... set to high
	cbi	DDRB, SIGNAL_L	; left input - no pullup
	lsl	BUTTON
	tst	LEFT
	brne	ignore_button	; can't read button when PB1 input is high
	sbic	PINB, SIGNAL_L
	inc	BUTTON		; shift button state into BUTTON
ignore_button:

	; control LED
	sbi	DDRB, SIGNAL_L	; left "input" is output controlling the LED
	cbi	PORTB, SIGNAL_L
	sbrc	LED, 0
	sbi	PORTB, SIGNAL_L
	sbi	DDRB, SIGNAL_R	; right "input" is output ...
	cbi	PORTB, SIGNAL_R	; ... with low value

	out	SREG, SREG_SAVE	; restore SREG
	reti

; ------------------------------------------------------------
; Copyright & date stamp
; ------------------------------------------------------------
	.org	0x3d0, 0xff
	.asciz	"Copyright 2006, Dick Streefland"
	.asciz	"2006-02-13"
	.align	4, 0xff
